<?php
namespace Teknisa\CADEST\Services;

use Teknisa\CADEST\Factories\Ger01700UnidadeFactory;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Ger01700UnidadeService {
    protected $factory;
    protected $cripto;

    /**
     * Construtor do Service.
     *
     * @param Ger01700UnidadeFactory $factory
     */
    public function __construct($cripto, Ger01700UnidadeFactory $factory) {
        $this->factory = $factory;
        $this->cripto  = $cripto;
    }

    /**
     * Procedimento que efetua a atualização dos produtos
     * para uma unidade de acordo com os dados informados
     *
     * @throws \Exception
     *
     * @param string $CDFILIAL
     * @param int $CDNVPRODUTO
     * @param int $NRORG
     */
    public function atualizaProduto($CDFILIAL, $CDNVPRODUTO, $NRORG) {
        $factoryReturn = $this->factory->getIDCTRLESTQ($CDFILIAL, $NRORG);

        if(count($factoryReturn) != 1)
            throw new \Exception("Erro Catastrófico! Erro na tabela PARAMFILIAL na unidade ".$CDFILIAL." da organização ".$NRORG."!", 1);

        $IDCTRLESTQ = $factoryReturn[0]["IDCTRLESTQ"];

        $factoryReturn = $this->factory->getIDUTILLOTE($CDFILIAL, $NRORG);
        if(count($factoryReturn) != 1)
            throw new \Exception("Erro Catastrófico! Erro na tabela PARAMFILIAL na unidade ".$CDFILIAL." da organização ".$NRORG."!", 1);

        $IDUTILLOTE = $factoryReturn[0]["IDUTILLOTE"];

        $this->factory->executeAtualizaProduto($CDFILIAL, $CDNVPRODUTO, $IDCTRLESTQ, $IDUTILLOTE);
    }

    /**
     * Função que retorna as validações de endereço de
     * uma unidade.
     *
     * @throws \Exception
     *
     * @param string $CDFILIAL
     * @param int $NRORG
     *
     * @return array
     */
    public function validateEndefili($CDFILIAL, $NRORG) {
        $addressTypes = array('C', 'E', 'P', 'T', 'X');

        $validation = array();

        if($CDFILIAL != 'IGNORE') {
            $factoryReturn = $this->factory->getValidationForEndefili($CDFILIAL, $NRORG);
            if(count($factoryReturn) != 1)
                throw new \Exception("Erro Catastrófico! Erro na validação do endereço da unidade ".$CDFILIAL." da organização ".$NRORG."!", 1);

            $validation['CDFILIAL']     = $factoryReturn[0]['CDFILIAL'];
            $validation['ALLTYPES']     = $addressTypes;
            $validation['USEDTYPES']    = explode('.', $factoryReturn[0]['CDROWS']);
            $validation['FREETYPES']    = array_values(array_diff($validation['ALLTYPES'], $validation['USEDTYPES']));
            $validation['MAYINSERT']    = count($validation['FREETYPES']) > 0;
        }

        $factoryReturn = $this->factory->getValidationForEndefiliFromParamgeral($NRORG);

        if(count($factoryReturn) != 1)
            throw new \Exception("Erro Catastrófico! Erro na tabela PARAMGERAL!", 1);

        $validation['REQENDFIELDS'] = ($factoryReturn[0]['IDUTENDMODFISC'] == 'S');

        return $validation;
    }

    /**
     * Função que cria campos conforme necessário para a
     * inserção/edição de Endefili.
     *
     * @param array $row
     *
     * @return array
     */
    public function validateEndefiliRow($row) {
        #concats address description
        $row['DSENDEFILI'] = ((array_key_exists('DSENDEFILISEP', $row) && array_key_exists('NRENDEFILISEP', $row))? $row['DSENDEFILISEP'].', '.$row['NRENDEFILISEP']: '');

        #concats numbers
        $row['NRFAXFILI'] = ((array_key_exists('NRDDDFAXFILI', $row) && array_key_exists('NRFAXFILISEP', $row))? $row['NRDDDFAXFILI'].$row['NRFAXFILISEP']: '');
        $row['NRTELEFILI'] = ((array_key_exists('NRDDDTELEFILI', $row) && array_key_exists('NRTELEFILISEP', $row))? $row['NRDDDTELEFILI'].$row['NRTELEFILISEP']: '');
        $row['NRTELEXFILI'] = ((array_key_exists('NRDDDTELEXFILI', $row) && array_key_exists('NRTELEXFILISEP', $row))? $row['NRDDDTELEXFILI'].$row['NRTELEXFILISEP']: '');
        

        $row['CDSENHEMAILF'] = $this->cripto->cripto($row['CDSENHEMAILF'], 2);
        
        #reduces neighborhood name size, if needed
        $row['NMBAIRFILI'] = (array_key_exists('NMBAIRFILI', $row))? substr($row['NMBAIRFILI'], 0, 30) : null;

        return($row);
    }
    
    /**
     * @param $password
     * @param bool $isEncrypt
     * @return mixed
     * @throws \Exception
     */
    public static function getDelphiCryptoUncripto($password, $isEncrypt = true){
        $url = self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';
        
        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar('2', XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema") // chave (salt)
        );
        //2
        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUnCripto';
        $params = array(
            'rdm'    => 'Bib.rdmDBGeral',
            'method' => $cryptType,
            'args'   => $args,
        );

            
        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = isset($result->executeCOMMethodResult)? $result->executeCOMMethodResult : null;
        } catch (\SoapFault $e) {
            return 'SoapError';
        }

        return $encryption;
    }

    /**
     * Função que verifica a tabela PARAMGERAL para
     * checar a parametrização de uso de parâmetros de
     * unidade padrão.
     *
     * @throws \Exception
     *
     * @return array
     */
    public function validateParamfilipadr($NRORG) {
        $factoryReturn = $this->factory->getValidationForParamfilipadr($NRORG);

        if(count($factoryReturn) != 1)
            throw new \Exception("Erro Catastrófico! Erro na tabela PARAMGERAL!", 1);

        $validation                 = array();
        $validation['NRORG']        = $factoryReturn[0]['NRORG'];
        $validation['SHOWPARAMTAB'] = ($factoryReturn[0]['IDSHOWPARAMS'] == 'S');

        return $validation;
    }

    /**
     * Função que cria campos conforme necessário para a
     * inserção de Unidade.
     *
     * @param array $row
     *
     * @return array
     */
    public function validateUnidadeNewRow($row) {
        try{
            if(!array_key_exists('IDALMOXINVAENTR', $row))
                $row['IDALMOXINVAENTR'] = 'N';
    
            if(!array_key_exists('IDCTRLESTQ', $row))
                $row['IDCTRLESTQ'] = 'N';
    
            return($row);
        } catch (\Exception $e){
            
        }
    }


    /**
     * Função que retorna as validações da primary key de
     * uma unidade.
     *
     * @throws \Exception
     *
     * @param string $CDFILIAL
     * @param int $NRORG
     *
     * @return array
     */
    public function validateUnidadePK($CDFILIAL, $NRORG) {
        $factoryReturn = $this->factory->getValidationForUnidadePK($CDFILIAL, $NRORG);

        if(count($factoryReturn) != 1)
            throw new \Exception("Erro Catastrófico! Erro na requisição de validação de código de unidade!", 1);

        $validation          = array();
        $validation['NRORG'] = $factoryReturn[0]['NRORG'];
        $validation['CODE']  = $factoryReturn[0]['CDFILIAL'];
        $validation['VALID'] = ($factoryReturn[0]['IDFREECODE'] == 'S');

        return $validation;
    }

    /**
     * Função que cria campos conforme necessário para a
     * inserção/edição de Unidade.
     *
     * @param array $row
     *
     * @return array
     */
    public function validateUnidadeRow($row) {
        if(!array_key_exists('NMRAZSOCFILI', $row) || !$row['NMRAZSOCFILI'])
            $row['NMRAZSOCFILI'] = $row['NMFILIAL'];

        if(!array_key_exists('CDALMOXARIFE', $row))
            $row['CDALMOXARIFE'] = ' ';
        if(!array_key_exists('DSALMOXARIFE', $row))
            $row['DSALMOXARIFE'] = ' ';

        if(!array_key_exists('CDLOCALESTOQ', $row))
            $row['CDLOCALESTOQ'] = ' ';
        if(!array_key_exists('DSLOCALESTOQ', $row))
            $row['DSLOCALESTOQ'] = ' ';
        
        if(isset($row["IDFUNCAOFILI"]) && $row["IDFUNCAOFILI"] == 'F') {
            $existeFilial = $this->factory->validaExisteFilialFranqueadora($row["CDFILIAL"]);
            if($existeFilial) {
                $mensagemErro = "Não foi possivel salvar a unidade como franqueadora. Unidade: ". $existeFilial["CDFILIAL"]
                                . " " . $existeFilial["NMFILIAL"] . " foi informada como franqueadora !";
                                
                throw new \Exception($mensagemErro);
            } 
        }
        
        return($row);
    }
    
    public function filialReplicaCliente($row) {
        $existeCliente = $this->factory->existeCliente($row["CDFILIAL"]);
        if(!$existeCliente) {
            $this->factory->insereCLiente($row["CDFILIAL"]);
            $this->factory->insereEndeClie($row["CDFILIAL"]);
        }
        else {
            throw new \Exception("Cliente já existe!");   
        }
    }
    
    public function atualizaAbaNfe($row) {
        $this->factory->atualizaUnidadeNfe($row["IDCOMPARQXMLFIL"], $row["CDFILIAL"]);
        // $this->factory->atualizaEnderecoNfe($row["DSEMAILEMXMLNFE"], $row["CDFILIAL"]);
    }
    
     public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }
}